/* ==========================================================================*/
/* - - - - - - - - - - - - -   M A I N . C   - - - - - - - - - - - - - - - - */
/* ==========================================================================*/
/*	Created   06-12-2004   M. Wheatley                                       */
/*	Modified  08-28-2004   MW                                                */
/*.........................................................................  */
/* Program to implment a WWV based Cuckoo Clock using DSPic Development board*/
/*.........................................................................  */
#define __dsPIC30F6014__
/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#include <p30f6014.h>
#include <dsp.h>
#include "common.h"
#include "lcd.h"
#include "menu.h"
#include "codec.h"
#include "atod.h"
#include "timers.h"
#include "gettime.h"
#include "procsignal.h"
#include "soundgen.h"

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                             ---------                                     */
/*                            | M a i n |                                    */
/*                             ---------                                     */
/*    This is the Main program entry point from boot.                        */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
int main(void)
{
int max,maxpos;
	PORTD = 0xFFFF;			//All LEDs off
	TRISDbits.TRISD0 = 0;	//LED output bits
	TRISDbits.TRISD1 = 0;
	TRISDbits.TRISD2 = 0;
	TRISDbits.TRISD3 = 0;
	InitTimers();
	TimeDelay(20);	//let system reset
	InitLcd();
	InitMenu();
	InitAD();
	InitCodec();
	InitProcSignal();
	InitGetTime();
	InitSoundGen();
	while(1)	//the one and only process loop
	{
		ServiceMenu();	//services the menu operation
		ServiceLCD();	//called to service the lcd when plotting
 		if(SemA.CodecTxDataRdy)	//if got a full buffer to process
		{//max 3.1mSec execution @ 8xPLL
			SemA.CodecTxDataRdy = FALSE;
			//pointer to empty half of transmit Queue
			ServiceSound( pActiveTxBuf );
		}
		else if(SemA.CodecRxDataRdy)
		{//max 2.5mSec execution @ 4xPLL
			SemA.CodecRxDataRdy = FALSE;
			//pointer to full half of receive Queue
			max = VectorMax(PROC_BLOCK_SIZE, pActiveRxBuf, &maxpos);
			if(max>30000)	//blink LED1 if close to peak
				LED1_ON
			else
				LED1_OFF
			ProcInput( pActiveRxBuf);
			if(CurrentMenu >= MENU_SIGNAL) 
				PlotTime(pActiveRxBuf);
			PlotPendulum();		//draw pendulum
		}
		else	//run processes when no codec data buffers are ready
		{//max 780uSec execution @ 4xPLL
	 		if(SemA.SecRdy)	//process at 1 second rate
			{
				FindSecSync();
		 		if(SemA.SecLocked)
				{
					FindMinSync();
					ProcDataBit();
				}
				IncTime();
				DisplayTime();
				SemA.SecRdy = FALSE;
			}
		}
	}
}

